<?php
namespace App\Controller\Component;

use Cake\Controller\Component;
use Cake\Network\Http\Client;
use Cake\Utility\Hash;
use Cake\Auth\DigestAuthenticate;

class ComptoirServerComponent extends Component
{

    public function initialize(array $config)
    {
        parent::initialize($config); // TODO: Change the autogenerated stub
        $this->options = [
            'type' => 'json',
            "headers" => [
                "Accept-Language"=>$this->request->param('language')]
        ];
        if ($this->_registry->getController()->Cookie->check("Digest")) {
            $digestString = $this->_registry->getController()->Cookie->read("Digest");
            $this->auth = new DigestAuthenticate($this->_registry, [
//                'realm' => 'localhost',
            ]);
            $this->options +=
                [
                    'auth' => [
                        'type' => 'digest',
                    ],
                ];

            $this->options['auth']  += $this->auth->parseAuthData($digestString['digest']);
            $this->options['auth']  += $this->_registry->getController()->Cookie->read("User");
        }

    }

    /**
     *
     * @param $url The url to get
     * @param $varname the var name to set or view
     * @param $path the path to extract data contains in the reponse
     */
    public function get( $url, $varname, $path )
    {

        try {
            $http = new Client();
            $response = $http->get($url,
                [],
                $this->options
                );
            $result = $response->body('json_decode');

            // TODO: vérifier que statusCode soit 200
            $this->_registry->getController()->set( $varname, Hash::get((array)$result, $path));
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
        return $result;
    }

    /**
     * @param $url
     * @param $varname
     * @param $path
     * @param array $data
     */
    public function post( $url, $varname, $path,$data=[] )
    {
        try {
            $http = new Client();
            $response = $http->post( $url, json_encode($data), $this->options );
            $result = $response->body('json_decode');
            if ($response->code == "200"){
                $this->_registry->getController()->set( $varname, Hash::get((array)$result, $path));
            }
            else{
                $this->_registry->getController()->Flash->error("Error");
               // return $this->_registry->getController()->redirect(['controller'=> 'Users','action' => 'add']);
            }
            return $result;
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
    }

    public function delete( $url, $varname, $path,$data=[] )
    {
        try {
            $http = new Client();
            $response = $http->delete( $url, json_encode($data), $this->options );
            $result = $response->body('json_decode');
            if ($response->code == "200"){
                $this->_registry->getController()->set( $varname, Hash::get((array)$result, $path));
            }
            else{
                $this->_registry->getController()->Flash->error("Error");
                // return $this->_registry->getController()->redirect(['controller'=> 'Users','action' => 'add']);
            }
            return $result;
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
    }


    /**
     * @param $url
     * @param $varname
     * @param $path
     * @param array $data
     */
    public function put( $url, $varname, $path,$data=[] )
    {
        try {
            $http = new Client();
            $response = $http->put( $url, json_encode($data), $this->options );
            $result = $response->body('json_decode');
            if ($response->code == "200"){
                $this->_registry->getController()->set( $varname, Hash::get((array)$result, $path));

            }
            else{
                $this->_registry->getController()->Flash->error("Error");
               // return $this->_registry->getController()->redirect(['controller'=> 'Users','action' => 'add']);
            }
            return $result;
        } catch (Exception $exception) {
            $this->log(var_export($exception, true), "debug");
        }
    }
}